/* lmpfuncs.c
 *
 * Copyright (C) 1999-2000 Bernhard Pietsch <bjtp@gmx.net>
 *
 * This file is part of LMP library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "lmp.h"
#include "lmpmath.h"

lmp_node_t * lmp_ctrl_int (lmp_node_t * var, lmp_node_t * root) {
  lmp_node_t * res = NULL, * p;

  //  lmp_warning ("[dbg] ->lmp_ctrl_int %s\n", lmp_infix_from_tree (root, 0));

  // Int f () dx = f () * x, as long as f does not depend on x.
  if (!lmp_tree_contains (root, var)) {
    res = lmp_node_new (OP, LMP_MULTIPLICATION, 2,
			 lmp_node_copy (var),
			 lmp_node_copy (root));
  }
  else if (root->what == VAR) {
    // root = var
    res = lmp_node_new (OP, LMP_DIVISION, 2,
			 lmp_node_new (OP, LMP_POWER, 2,
				       lmp_node_copy (root),
				       lmp_node_new (VAL, 2*ONE)),
			 NEW_VAL (2*ONE));
  }
  else if (root->what == OP) {
    switch (root->data.operation.op) {
    case LMP_MULTIPLICATION:
      {
	lmp_node_t
	  * l = root->data.operation.argv [0],
	  * r = root->data.operation.argv [1];
	lmp_stack_t
	  * exp  = lmp_stack_new (),
	  * fact = lmp_stack_new ();

	if (!lmp_tree_contains (l, var))
	  res = lmp_node_new (OP, LMP_MULTIPLICATION, 2,
			       lmp_node_copy (l),
			       lmp_ctrl_int (var, r));
	else if (!lmp_tree_contains (r, var))
	  res = lmp_node_new (OP, LMP_MULTIPLICATION, 2,
			       lmp_node_copy (r),
			       lmp_ctrl_int (var, l));

	else if (lmp_node_polynom (l, var, fact, exp)) {
	  lmp_node_t * int_left = lmp_ctrl_int (var, r);

	  if (int_left->what != VAL || int_left->data.value == LMP_INT_NO_ROOT)
	    res = lmp_node_new (OP, LMP_SUBTRACTION, 2,
				lmp_node_new (OP, LMP_MULTIPLICATION, 2,
					      lmp_node_copy (l),
					      lmp_node_copy (int_left)),
				lmp_ctrl_int (var,
					      lmp_node_new (OP, LMP_MULTIPLICATION, 2,
							    lmp_node_copy (int_left),
							    lmp_ctrl_diff (var, l))));

	  lmp_node_free (int_left);
	}
	else if (lmp_node_polynom (r, var, fact, exp)) {
	  lmp_node_t * int_left = lmp_ctrl_int (var, l);

	  if (int_left->what != VAL || int_left->data.value == LMP_INT_NO_ROOT)
	    res = lmp_node_new (OP, LMP_SUBTRACTION, 2,
				lmp_node_new (OP, LMP_MULTIPLICATION, 2,
					      lmp_node_copy (r),
					      lmp_node_copy (int_left)),
				lmp_ctrl_int (var,
					      lmp_node_new (OP, LMP_MULTIPLICATION, 2,
							    lmp_node_copy (int_left),
							    lmp_ctrl_diff (var, r))));
	  lmp_node_free (int_left);
	}

	lmp_stack_free (exp);
	lmp_stack_free (fact);
      }
      break;
    case LMP_DIVISION:
      {
	lmp_node_t
	  * l = root->data.operation.argv [0],
	  * r = root->data.operation.argv [1];
	  //	  * fact = NULL,
	  //	  * exp = NULL;
	lmp_stack_t
	  * exp  = lmp_stack_new (),
	  * fact = lmp_stack_new ();

	if (!lmp_tree_contains (r, var))
	  res = lmp_node_new (OP, LMP_DIVISION, 2,
			      lmp_ctrl_int (var, l),
			      lmp_node_copy (r));
	else if (lmp_node_polynom (r, var, fact, exp)) {
	  lmp_node_t * tmp;
	  int i = exp->si;

	  res = lmp_node_create_polynom_fragment (var, lmp_stack_get (fact, exp->si-i), lmp_node_new (OP, LMP_UNARY_MINUS, 1, lmp_stack_get (exp, exp->si-i)));

	  while (i > 1) {
	    i--;
	    res = lmp_node_new (OP, LMP_ADDITION, 2,
				lmp_node_create_polynom_fragment (var, lmp_stack_get (fact, exp->si-i), lmp_node_new (OP, LMP_UNARY_MINUS, 1, lmp_stack_get (exp, exp->si-i))),
				res);
	  }

	  tmp = lmp_node_new (OP, LMP_MULTIPLICATION, 2,
			      lmp_node_copy (l),
			      res);
	  res = lmp_ctrl_int (var, tmp);
	  lmp_node_free (tmp);

	}
	/*
	else if (lmp_node_polynom_fragment (r, var, &fact, &exp)) {
	  lmp_node_t * tmp2, * tmp;

	  tmp = lmp_node_new (OP, LMP_MULTIPLICATION, 2,
				  lmp_node_new (OP, LMP_POWER, 2,
						lmp_node_copy (var),
						lmp_node_new (OP, LMP_MULTIPLICATION, 2,
							      NEW_VAL (-1*ONE),
							      exp)),
				  lmp_node_copy (l));

	  tmp2 = tmp;
	  tmp = lmp_tree_simplify_full (tmp);
	  lmp_node_free (tmp2);

	  res = lmp_node_new (OP, LMP_DIVISION, 2,
			      lmp_ctrl_int (var, tmp),
			      fact);
	  lmp_node_free (tmp);
	}
	else {
	  lmp_node_free (fact);
	  lmp_node_free (exp);
	}
	*/
	lmp_stack_free (fact);
	lmp_stack_free (exp);
      }
      break;
    case LMP_ADDITION:
      res = lmp_node_new (OP, LMP_ADDITION, 2,
			   lmp_ctrl_int (var, root->data.operation.argv [0]),
			   lmp_ctrl_int (var, root->data.operation.argv [1]));
      break;
    case LMP_SUBTRACTION:
      res = lmp_node_new (OP, LMP_SUBTRACTION, 2,
			   lmp_ctrl_int (var, root->data.operation.argv [0]),
			   lmp_ctrl_int (var, root->data.operation.argv [1]));
      break;
    case LMP_POWER:
      {
	lmp_node_t
	  * l = root->data.operation.argv [0],
	  * r = root->data.operation.argv [1];

	if (l->what == VAR && !lmp_tree_contains (r, var)) {
	  if (!strcmp (l->data.variable, var->data.variable)) {
	    if (lmp_tree_evaluable (r) && lmp_eval (r) == (lmp_value_type) -1.0) {
	      res = lmp_node_new (OP, LMP_LOGARITHM, 1, lmp_node_copy (l));
	    }
	    else {
	      res = lmp_node_new (OP, LMP_DIVISION, 2,
				  lmp_node_new (OP, LMP_POWER, 2,
						lmp_node_copy (l),
						lmp_node_new (OP, LMP_ADDITION, 2,
							      lmp_node_copy (r),
							      NEW_VAL (ONE))),
				  lmp_node_new (OP, LMP_ADDITION, 2,
						lmp_node_copy (r),
						NEW_VAL (ONE)));
	    }
	  }
	}
      }
      break;
    case LMP_SINE:
      {
	lmp_node_t * arg = root->data.operation.argv [0];
	lmp_node_t * fact;

	if (arg->what == VAR)
	  res = lmp_node_new (OP, LMP_MULTIPLICATION, 2,
			      NEW_VAL (-1*ONE),
			      lmp_node_new (OP, LMP_COSINE, 1,
					    lmp_node_copy (arg)));
	else if (lmp_node_factored (arg, var, &fact))
	  res = lmp_node_new (OP, LMP_MULTIPLICATION, 2,
			      lmp_node_new (OP, LMP_DIVISION, 2,
					    NEW_VAL (-1*ONE),
					    fact),
			      lmp_node_new (OP, LMP_COSINE, 1,
					    lmp_node_copy (arg)));
      }
      break;
    case LMP_COSINE:
      {
	lmp_node_t * arg = root->data.operation.argv [0];
	lmp_node_t * fact;

	if (arg->what == VAR)
	  res = lmp_node_new (OP, LMP_COSINE, 1,
			      lmp_node_copy (arg));
	else if (lmp_node_factored (arg, var, &fact))
	  res = lmp_node_new (OP, LMP_DIVISION, 2,
			      lmp_node_new (OP, LMP_SINE, 1,
					    lmp_node_copy (arg)),
			      fact);
      }
      break;
    case LMP_LOGARITHM:
      {
	lmp_node_t * arg = root->data.operation.argv [0];
	lmp_node_t * fact;

	if (lmp_node_factored (arg, var, &fact))
	  res = lmp_node_new (OP, LMP_SUBTRACTION, 2,
			      lmp_node_new (OP, LMP_MULTIPLICATION, 2,
					    lmp_node_copy (var),
					    lmp_node_copy (root)),
			      lmp_node_copy (var));
      }
      break;
    case LMP_UNARY_MINUS:
      res = lmp_node_new (OP, LMP_UNARY_MINUS, 1,
			  lmp_ctrl_int (var, root->data.operation.argv [0]));
    }
  }


  if (!res) {
    lmp_error ("Integration failed: no root found: %s\n", lmp_infix_from_tree (root, 0));
    res = NEW_VAL (LMP_INT_NO_ROOT);
  }

  p = res;
  res = lmp_tree_simplify_full (res);
  lmp_node_free (p);

  //  lmp_warning ("[dbg] <-lmp_ctrl_int %s\n", lmp_infix_from_tree (res, 0));
  return res;  
}

lmp_node_t * lmp_ctrl_diff (lmp_node_t * var, lmp_node_t * root) {
  lmp_node_t * p, * res;

  if (var->what != VAR)
    lmp_error ("Argument %d is not a variable\n", lmp_token_args ());

  p = root;
  if (p) {
    switch (root->what) {
    case OP:
      switch (root->data.operation.op) {
      case LMP_MULTIPLICATION:
        res = lmp_node_new 
	  (OP, LMP_ADDITION, 2,
	   lmp_node_new 
	   (OP, LMP_MULTIPLICATION, 2,
	    lmp_ctrl_diff (var, p->data.operation.argv[0]),
	    lmp_node_copy (p->data.operation.argv[1])),
	   lmp_node_new 
	   (OP, LMP_MULTIPLICATION, 2,
	    lmp_ctrl_diff (var, p->data.operation.argv[1]),
	    lmp_node_copy (p->data.operation.argv[0])));
	break;
      case LMP_DIVISION:
	{

	  lmp_node_t
	    * ld = lmp_ctrl_diff (var, p->data.operation.argv [0]);

	  if (!lmp_tree_contains (p->data.operation.argv [1], var))
	    res = lmp_node_new (OP, LMP_MULTIPLICATION, 2,
				lmp_node_new (OP, LMP_DIVISION, 2,
					      NEW_VAL (ONE),
					      lmp_node_copy (p->data.operation.argv [1])),
				ld);
	  else
	    res = lmp_node_new (OP, LMP_DIVISION, 2,
				lmp_node_new (OP, LMP_SUBTRACTION, 2,
					      lmp_node_new (OP, LMP_MULTIPLICATION, 2,
							    lmp_node_copy (p->data.operation.argv [1]),
							    ld),
					      lmp_node_new (OP, LMP_MULTIPLICATION, 2,
							    lmp_node_copy (p->data.operation.argv [0]),
							    lmp_ctrl_diff (var, p->data.operation.argv [1]))),
				lmp_node_new (OP, LMP_POWER, 2,
					      lmp_node_copy (p->data.operation.argv [1]),
					      NEW_VAL (2*ONE)));
	}
	break;
      case LMP_POWER:
	if (lmp_tree_contains (p->data.operation.argv [1], var)) {
	  if (lmp_tree_contains (p->data.operation.argv [0], var))
	    res = lmp_ctrl_diff 
	      (var, lmp_node_new 
	       (OP, LMP_EXP, 1,
		lmp_node_new (OP, LMP_MULTIPLICATION, 2,
			      lmp_node_copy (p->data.operation.argv[1]),
			      lmp_node_new (OP, LMP_LOGARITHM, 1,
					    lmp_node_copy (p->data.operation.argv[0])))));
	  else
	    res = lmp_node_new 
	      (OP, LMP_MULTIPLICATION, 2,
	       lmp_node_new
	       (OP, LMP_MULTIPLICATION, 2,
		lmp_node_copy (p),
		lmp_node_new (OP, LMP_LOGARITHM, 1,
			      lmp_node_copy (p->data.operation.argv [0]))),
	       lmp_ctrl_diff (var, p->data.operation.argv [1]));
	}
	else
	  res = lmp_node_new
	    (OP, LMP_MULTIPLICATION, 2,
	     lmp_node_new
	     (OP, LMP_MULTIPLICATION, 2,
	      lmp_node_new 
	      (OP, LMP_POWER, 2,
	       lmp_node_copy (p->data.operation.argv [0]),
	       lmp_node_new (OP, LMP_SUBTRACTION, 2,
			     lmp_node_copy (p->data.operation.argv [1]),
			     NEW_VAL (ONE))),
	      lmp_ctrl_diff (var, p->data.operation.argv [0])),
	     lmp_node_copy (p->data.operation.argv [1]));
	break;
      case LMP_EXP:
	res = lmp_node_new (OP, LMP_MULTIPLICATION, 2,
		 lmp_node_new (OP, LMP_EXP, 1,
		    lmp_node_copy (p->data.operation.argv[0])),
		    lmp_ctrl_diff (var, p->data.operation.argv[0])
	       );
	break;
      case LMP_LOG:
	res = lmp_node_new
	  (OP, LMP_MULTIPLICATION, 2,
	   lmp_node_new (OP, LMP_DIVISION, 2,
			 NEW_VAL (ONE),
			 lmp_node_new 
			 (OP, LMP_MULTIPLICATION, 2,
			  lmp_node_copy (p->data.operation.argv [0]),
			  lmp_node_new 
			  (OP, LMP_LOGARITHM, 1, lmp_node_copy (p->data.operation.argv [1])))),
	   lmp_ctrl_diff (var, p->data.operation.argv [0]));
	break;
      case LMP_SINE:
	res = lmp_node_new (OP, LMP_MULTIPLICATION, 2,
			    lmp_node_new (OP, LMP_COSINE, 1,
					  lmp_node_copy (p->data.operation.argv[0])),
			    lmp_ctrl_diff (var, p->data.operation.argv[0]));
	break;
      case LMP_COSINE:
	res = lmp_node_new 
	  (OP, LMP_MULTIPLICATION, 2,
	   lmp_node_new (OP, LMP_UNARY_MINUS, 1,
			 lmp_node_new (OP, LMP_SINE, 1,
				       lmp_node_copy (p->data.operation.argv[0]))),
	   lmp_ctrl_diff (var, p->data.operation.argv [0]));
	break;
      case LMP_TANGENT:
	res = lmp_node_new
	  (OP, LMP_MULTIPLICATION, 2,
	   lmp_node_new
	   (OP, LMP_ADDITION, 2,
	    NEW_VAL (ONE),
	    lmp_node_new 
	    (OP, LMP_POWER, 2,
	     NEW_VAL (ONE*2),
	     lmp_node_new (OP, LMP_TANGENT, 1,
			   lmp_node_copy (p->data.operation.argv[0])))),
	   lmp_ctrl_diff (var, p->data.operation.argv [0]));
	break;
      case LMP_COTANGENT:
	res = lmp_node_new
	  (OP, LMP_MULTIPLICATION, 2,
	   lmp_node_new (OP, LMP_SUBTRACTION, 2,
			 NEW_VAL (-1*ONE),
			 lmp_node_new
			 (OP, LMP_POWER, 2,
			  lmp_node_copy (p),
			  NEW_VAL (2*ONE))),
	   lmp_ctrl_diff (var, p->data.operation.argv [0]));
	break;
      case LMP_ARC_SINE:
      case LMP_ARC_COSINE:
	res = lmp_node_new
	  (OP, LMP_MULTIPLICATION, 2,
	   lmp_node_new
	   (OP, LMP_DIVISION, 2,
	    NEW_VAL (ONE),
	    lmp_node_new (OP, LMP_SQRT, 1,
			  lmp_node_new (OP, LMP_SUBTRACTION, 2,
					NEW_VAL (ONE),
					lmp_node_new (OP, LMP_POWER, 2,
						      lmp_node_copy (p->data.operation.argv[0]),
						      NEW_VAL (2*ONE))))),
	   lmp_ctrl_diff (var, p->data.operation.argv [0]));
	break;
      case LMP_ARC_TANGENT:
	res = lmp_node_new
	  (OP, LMP_MULTIPLICATION, 2,
	   lmp_node_new
	   (OP, LMP_DIVISION, 2,
	    NEW_VAL (ONE),
	    lmp_node_new (OP, LMP_ADDITION, 2,
			  lmp_node_new (VAL, ONE),
			  lmp_node_new (OP, LMP_POWER, 2,
					lmp_node_copy (p->data.operation.argv[0]),
					NEW_VAL (2*ONE)))),
	   lmp_ctrl_diff (var, p->data.operation.argv [0]));
	break;
      case LMP_LOGARITHM:
	res = lmp_node_new
	  (OP, LMP_DIVISION, 2,
	   lmp_ctrl_diff (var, p->data.operation.argv [0]),
	   lmp_node_copy (p->data.operation.argv [0]));
	break;
      case LMP_LOG_10:
	res = lmp_node_new
	  (OP, LMP_MULTIPLICATION, 2,
	   lmp_node_new (OP, LMP_DIVISION, 2,
			 NEW_VAL (ONE),
			 lmp_node_new
			 (OP, LMP_MULTIPLICATION, 2,
			  lmp_node_copy (p->data.operation.argv [0]),
			  NEW_VAL (LMP_CONST_LN10*ONE))),
	   lmp_ctrl_diff (var, p->data.operation.argv [0]));
	break;
      case LMP_SQRT:
	res = lmp_node_new 
	  (OP, LMP_DIVISION, 2,
	   lmp_ctrl_diff (var, p->data.operation.argv [0]),
	   lmp_node_new
	   (OP, LMP_MULTIPLICATION, 2,
	    NEW_VAL (2*ONE),
	    lmp_node_new (OP, LMP_SQRT, 1,
			  lmp_node_copy (p->data.operation.argv [0]))));
	break;
      case LMP_UNARY_MINUS:
	res = lmp_node_new (OP, p->data.operation.op, 1,
			    lmp_ctrl_diff (var, p->data.operation.argv [0]));
	break;
      default :
	res = lmp_node_new (OP, p->data.operation.op, 2,
			     lmp_ctrl_diff (var, p->data.operation.argv[0]),
			     lmp_ctrl_diff (var, p->data.operation.argv[1]));
      }
      break;
    case CONTROL:
      res = lmp_ctrl_diff (var, lmp_proc_control (p));
      break;
    case VAL:
      res = NEW_VAL (ZERO);
      break;
    case VAR:
      if (!strcmp (p->data.variable, var->data.variable))
        res = NEW_VAL (ONE);
      else
        res = NEW_VAL (ZERO);
      break;
    }
  }

  p = res;
  res = lmp_tree_simplify_full (res);
  lmp_node_free (p);

  return res;
}

lmp_node_t * lmp_ctrl_sum (lmp_node_t * var, lmp_node_t * start, lmp_node_t * end, lmp_node_t * root) {
  lmp_value_type cur;
  lmp_node_t * res;

  if (var->what != VAR)
    lmp_error ("Argument %d is not a variable\n", lmp_token_args ());

  cur = lmp_eval (start);
  lmp_vt_add (var->data.variable, cur);
  res = root;
  if (CREAL (cur) <= CREAL (lmp_eval (end))) {
    res = lmp_tree_var_eval (var, root);
    return lmp_node_new (OP, LMP_ADDITION, 2, res, lmp_ctrl_sum (var, NEW_VAL (cur + 1.0), end, root));
  }

  return NEW_VAL (ZERO);
}

lmp_node_t * lmp_ctrl_prod (lmp_node_t * var, lmp_node_t * start, lmp_node_t * end, lmp_node_t * root) {
  lmp_value_type cur;
  lmp_node_t * res;

  if (var->what != VAR)
    lmp_error ("Argument %d is not a variable\n", lmp_token_args ());

  cur = lmp_eval (start);
  lmp_vt_add (var->data.variable, cur);
  res = root;
  if (CREAL (cur) <= CREAL (lmp_eval (end))) {
    res = lmp_tree_var_eval (var, root);
    return lmp_node_new (OP, LMP_MULTIPLICATION, 2, res, lmp_ctrl_prod (var, NEW_VAL (cur + 1.0), end, root));
  }

  return NEW_VAL (ONE);
}
