/* lmphelp.c
 *
 * Copyright (C) 1999-2000 Bernhard Pietsch <bjtp@gmx.net>
 *
 * This file is part of LMP library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include "lmphelp.h"

/* Print usage if no arguments are available */
void lmp_print_usage () {
  printf ("L Math Processor v%s\n"
	  "Try `lmp --help' or the manpage for more information.\n", LMP_VERSION);
  exit (1);
}

/* Print LMP version */
void lmp_print_version () {
  printf ("%s\n", LMP_VERSION);
  exit (0);
}

/* Print lmp help page */
void lmp_print_help () {
  printf ("Usage: lmp [OPTIONS] EXPR | lmp --help | lmp --version\n\n"

	  "OPTIONS may be:\n\n"

	  "  -bal\n"
	  "  --balancing\n"
	  "\t\tThis is set by default. It resets the -nb flag.\n"
	  "  -jb\n"
	  "  --just-balancing\n"
	  "\t\tSame as -ns -bal\n"
	  "  -nb\n"
	  "  --no-balancing\n"
	  "\t\tDon't balance expression trees during simplification.\n"
	  "  -nc\n"
	  "  --no-computing\n"
	  "\t\tDon't compute, just scan and simplify terms.\n"
	  "  -np\n"
	  "  --no-scan-args\n"
	  "\t\tDon't scan each argument: If this flag is not defined, lmp will\n"
	  "\t\tscan each argument:\n\n"

	  "\t\t  lmp 'sin(3.141)^2'\n\n"

	  "\t\tIf you define -np, you will have to write\n\n"

	  "\t\t  lmp -np sin \\( 3.141 \\) ^ 2\n\n"

	  "\t\tYou may NOT specify more than one terms.\n"
	  "  -nps\n"
	  "  --no-post-simplify\n"
	  "\t\tDon't run last level simplification.\n"
	  "  -ns\n"
	  "  --no-simplify\n"
	  "\t\tDon't simplify terms before computing.\n"
	  "\t\tTogether with -nc LMP prints the scanned term.\n"
	  "  -qi\n"
	  "  --quiet-init\n"
	  "\t\tThis suppresses output of initialization results. Normally,\n"
	  "\t\t'x=1' or something would make lmp to compute the init term like\n"
	  "\t\tany other and so to output the result ('1').\n"
	  "  -pa\n"
	  "  --scan-args\n"
	  "\t\tThis is set by default. It resets the -np flag.\n"
	  "  -ps\n"
	  "  --post-simplify\n"
	  "\t\tThis is set by default. It resets the -nps flag.\n"
	  "  -pt\n"
	  "  --print-tree\n"
	  "\t\tShow expression as a tree. Can't be set in LMP_OPTIONS.\n"
	  "  -si\n"
	  "  --simplify\n"
	  "\t\tThis is set by default. It resets the -ns flag.\n"
	  "  -vi\n"
	  "  --verbose-init\n"
	  "\t\tThis is set by default. It resets the -qi flag.\n"
	  "  -vt INITSTR\n"
	  "  --vartable=INITSTR\n"
	  "\t\tDeclare variables: INITSTR is a string like \"x=-5:pi=3.141\".\n"
	  "\t\tYou may now use x or pi in your formulas. Lmp will replace all\n"
	  "\t\toccurances of x with -5 etc.\n\n"
	  "  -b BASE\n"
	  "  --base=BASE\n"
          "\t\tSpecifies the BASE of output (for integer values), e.g. -b 16\n"
	  "\t\twould output a hexadezimal number (-b 2 for binary output).\n"
	  "  -p PREC\n"
	  "  --precision=PREC\n"
	  "\t\tSpecifies the PRECision of the output, which means the number\n"
	  "\t\tof digits after the dot, or with specified base, the minimum\n"
	  "\t\tnumber of digits of the whole output, filled to this number\n"
	  "\t\twith zeros if necessary.\n"
	  "  -sp SIGPREC\n"
	  "  --significance=SIGPREC\n"
	  "\t\tSpecifies the SIGnificant PRECision of the output, which means\n"
	  "\t\tthe number of digits from the first non-zero digit to the last\n"
	  "\t\tnon-zero digit. If there more significant digits than specified,\n"
	  "\t\tthey are cut and not shown, else all significant digits are\n"
	  "\t\tshown.\n\n"

	  "\tIf none of the -sp and -p options are specified, the program\n"
	  "\tassumes -p 6. If -sp is specified and the result is smaller than 0.0001,\n"
	  "\tthe output is formatted in the <MANTISSE>e<EXPONENT> way, which means:\n\n"

	  "\t\t\tMANTISSE * 10 ^ EXPONENT\n\n"

	  "\tIf -sp is specified, -p won't change anything.\n\n"

	  "\t\tIf -b is specified, -sp won't change anything, but -p specifies\n"
	  "\tthe minimum number of digits of the output. If the result has less\n"
	  "\tdigits, the output is fullfilled with leading zeros.\n\n"

	  "EXPR may be:\n\n"

	  "  ARG1 or ARG2\tTrue as soon as one of both arguments is true.\n"
	  "  ARG1 and ARG2\tTrue if both arguments are true.\n\n"

	  "  ARG1 < ARG2\tTrue if ARG1 is less than ARG2.\n"
	  "  ARG1 <= ARG2\tTrue if ARG1 is less than or equal to ARG2.\n"
	  "  ARG1 > ARG2\tTrue if ARG1 is greater than ARG2.\n"
	  "  ARG1 >= ARG2\tTrue if ARG1 is greater than or equal to ARG2.\n"
	  "  ARG1 == ARG2\tTrue if ARG1 is equal to ARG2.\n"
	  "  ARG1 != ARG2\tTrue if ARG1 is unequal to ARG2.\n\n"

	  "  ARG1 + ARG2\tSum of ARG1 and ARG2.\n"
	  "  ARG1 - ARG2\tDifference of ARG1 and ARG2.\n\n"

	  "  ARG1 * ARG2\tProduct of ARG1 and ARG2.\n"
	  "  ARG1 / ARG2\tQuotient of ARG1 and ARG2.\n"
	  "  ARG1 div ARG2\tInteger quotient of ARG1 and ARG2.\n"
	  "  ARG1 mod ARG2\tRemainder of ARG1 divided by ARG2.\n\n"

	  "  ARG1 ^ ARG2\tARG1 raised to the power of ARG2.\n"
	  "  log BASE ARG\tBASE logarithm of ARG.\n\n"

	  "  sin ARG\tsine function of ARG\n"
	  "  cos ARG\tcosine function of ARG\n"
	  "  tan ARG\ttangent function of ARG\n"
	  "  cot ARG\tcotangent function of ARG\n"
	  "  arcsin ARG\tarcsine function of ARG\n"
	  "  arccos ARG\tarc cosine function of ARG\n"
	  "  arctan ARG\tarc tangent function of ARG\n"
	  "  exp ARG\texponential function of ARG\n"
	  "  sqrt ARG\tsquare root of ARG\n"
	  "  lg ARG\tBase-10 logarithm of ARG\n"
	  "  ln ARG\tNatural logarithm of ARG\n\n"

	  "  diff VAR FUNC\n"
	  "\t\tdifferentiation of FUNC with VAR\n"
	  "  int VAR FUNC\n"
	  "\t\troot function of of FUNC: Int FUNC d VAR\n"

	  "  sum INDEX FROM TO FUNC\n"
	  "\t\tSum FUNC with INDEX\n"
	  "  prod INDEX FROM TO FUNC\n"
	  "\t\tProduct of FUNC with INDEX\n\n"

	  "  ( EXPR )\tValue of EXPR.\n\n"

	  "  { REAL , IMAG }\tCreates a complex value.\n\n"

	  "Beware many operators (*, <, > etc) need to be protected from shells. They have\n"
	  "to be quoted ('*' for * etc) or escaped (\\* for * etc).\n");
  exit (0);
}
