#include "lmp.h"
#include "lmpmath.h"
#include <stdio.h>

char * lmp_output_op2latex (lmp_operator op) {
  switch (op) {
  case LMP_OUTER_OR:
    return "||";
  case LMP_OUTER_AND:
    return "\\&\\&";
  case LMP_EQUALS:
    return "==";
  case LMP_UNEQUALS:
    return "!=";
  case LMP_GREATER_EQUAL:
    return ">=";
  case LMP_LOWER_EQUAL:
    return "<=";
  case LMP_GREATER:
    return ">";
  case LMP_LOWER:
    return "<";
  case LMP_ADDITION:
    return "+";
  case LMP_SUBTRACTION:
    return "-";
  case LMP_MULTIPLICATION:
    return "\\cdot";
  case LMP_DIVISION:
    return "/";
  case LMP_MODULO:
    return "\\%";
  case LMP_DIV_NO_REM:
    return "\\mbox{ div }";
  case LMP_POWER:
    return "^";
  case LMP_LOG:
    return "\\log_";
  case LMP_SINE:
    return "\\sin";
  case LMP_COSINE:
    return "\\cos";
  case LMP_TANGENT:
    return "\\tan";
  case LMP_COTANGENT:
    return "\\cot";
  case LMP_ARC_SINE:
    return "\\arcsin";
  case LMP_ARC_COSINE:
    return "\\arccos";
  case LMP_ARC_TANGENT:
    return "\\arctan";
  case LMP_LOGARITHM:
    return "\\ln";
  case LMP_LOG_10:
    return "\\lg";
  case LMP_EXP:
    return "e^";
  case LMP_SQRT:
    return "sqrt";
  case LMP_UNARY_MINUS:
    return "-";
  }

  return NULL;
}

char * lmp_output_ctrl2latex (lmp_control op) {
  switch (op) {
  case LMP_DIFF:
    return "diff";
  case LMP_INT:
    return "\\int";
  case LMP_SUM:
    return "\\sum";
  case LMP_PROD:
    return "\\prod";
  }
  return NULL;
}

char * lmp_output_latex (lmp_node_t * root, int expand) {
  lmp_node_t * p = root;
  char * res;

  if (!p) {
    return "";
  }

  switch (root->what) {
  case OP:
    switch (root->data.operation.op) {
    case LMP_OUTER_OR:
    case LMP_OUTER_AND:
    case LMP_EQUALS:
    case LMP_UNEQUALS:
    case LMP_GREATER_EQUAL:
    case LMP_LOWER_EQUAL:
    case LMP_GREATER:
    case LMP_LOWER:
    case LMP_ADDITION:
    case LMP_SUBTRACTION:
    case LMP_MULTIPLICATION:
    case LMP_MODULO:
    case LMP_DIV_NO_REM:
      asprintf (&res, "\\left(%s %s %s\\right)",
		lmp_output_latex (p->data.operation.argv [0], expand),
		lmp_output_op2latex (p->data.operation.op),
		lmp_output_latex (p->data.operation.argv [1], expand));
      break;
    case LMP_DIVISION:
      asprintf (&res, "\\frac{%s}{%s}",
		lmp_output_latex (p->data.operation.argv [0], expand),
		lmp_output_latex (p->data.operation.argv [1], expand));
      break;
    case LMP_LOG:
      asprintf (&res, "\\log_{%s} %s",
		lmp_output_latex (p->data.operation.argv[1], expand),
		lmp_output_latex (p->data.operation.argv[0], expand));
      break;
    case LMP_POWER:
      asprintf (&res, "%s^{%s}",
		lmp_output_latex (p->data.operation.argv[0], expand),
		lmp_output_latex (p->data.operation.argv[1], expand));
      break;
    case LMP_SINE:
    case LMP_COSINE:
    case LMP_TANGENT:
    case LMP_COTANGENT:
    case LMP_ARC_SINE:
    case LMP_ARC_COSINE:
    case LMP_ARC_TANGENT:
    case LMP_LOGARITHM:
    case LMP_LOG_10:
    case LMP_UNARY_MINUS:
      asprintf (&res, "\\left(%s %s\\right)",
		lmp_output_op2latex (p->data.operation.op),
		lmp_output_latex (p->data.operation.argv[0], expand));
      break;
    case LMP_EXP:
      asprintf (&res, "e^{%s}",
		lmp_output_latex (p->data.operation.argv[0], expand));
      break;
    case LMP_SQRT:
      asprintf (&res, "\\sqrt{%s}",
		lmp_output_latex (p->data.operation.argv[0], expand));
      break;
    }
    break;
  case CONTROL:
    if (expand) {
      res = lmp_output_latex (lmp_proc_control (p), expand);
    }
    else
      switch (p->data.control.op) {
      case LMP_DIFF:
	asprintf (&res, "%s_{%s}'",
		  lmp_output_latex (p->data.control.argv [1], expand),
		  lmp_output_latex (p->data.control.argv [0], expand));
	break;
      case LMP_INT:
	asprintf (&res, "\\int %s \\mbox{ d}%s",
		  lmp_output_latex (p->data.control.argv [1], expand),
		  lmp_output_latex (p->data.control.argv [0], expand));
	break;
      case LMP_SUM:
      case LMP_PROD:
	asprintf (&res, "%s_{%s=%s}^{%s} %s",
		  lmp_output_ctrl2latex (p->data.control.op),
		  lmp_output_latex (p->data.control.argv [0], expand),
		  lmp_output_latex (p->data.control.argv [1], expand),
		  lmp_output_latex (p->data.control.argv [2], expand),
		  lmp_output_latex (p->data.control.argv [3], expand));
	break;
      }
    break;
  case VAL:
    if (IS_REAL (p->data.value))
      res = lmp_print_number_to_string (CREAL (p->data.value));
    else
      asprintf (&res, "%s + %si",
		lmp_print_number_to_string (CREAL (p->data.value)),
		lmp_print_number_to_string (CIMAG (p->data.value)));
    break;
  case VAR:
    asprintf (&res, "%s", p->data.variable);
    break;
  }

  return res;
}
