/* lmpstack.c
 *
 * Copyright (C) 1999-2000 Bernhard Pietsch <bjtp@gmx.net>
 *
 * This file is part of LMP library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdarg.h>
#include <malloc.h>
#include "lmp.h"

lmp_stack_t * lmp_stack_new () {
  lmp_stack_t * stack;

  stack = malloc (sizeof (lmp_stack_t));
  stack->si = 0;
  stack->bs = 16;
  stack->sp = (lmp_node_t **) malloc (sizeof (lmp_node_t *) * stack->bs);

  return stack;
}

#define ROUND(v,a) (v % a == 0 ? v+a : (v/a+1)*a)

lmp_stack_t * lmp_stack_malloc (int elems) {
  lmp_stack_t * stack;

  stack = lmp_stack_new ();
  if (elems >= stack->bs) {
    stack->sp = (lmp_node_t **) realloc (stack->sp, sizeof (lmp_node_t *) * (ROUND (elems, stack->bs)));
  }

  return stack;
}

int lmp_stack_free (lmp_stack_t * stack) {
  free (stack->sp);
  free (stack);

  return 1;
}

int lmp_stack_push (lmp_stack_t * stack, lmp_node_t * node) {
  stack->si += 1;

  if (stack->si % stack->bs == 0) {
    stack->sp = (lmp_node_t **) realloc (stack->sp, sizeof (lmp_node_t *) * (stack->si+stack->bs));
  }

  stack->sp [stack->si - 1] = node;

  return 1;
}

lmp_node_t * lmp_stack_get (lmp_stack_t * stack, int elem) {
  if (elem < stack->si) {
    return stack->sp [elem];
  }
  else {
    lmp_error ("Stack get failed: only %d elements on stack.\n", stack->si);
    return NULL;
  }
}

int lmp_stack_split (lmp_stack_t * stack, lmp_stack_t ** l, lmp_stack_t ** r) {
  int i;

  *r = lmp_stack_new ();
  *l = lmp_stack_new ();

  for (i = 0; i < stack->si/2; i++)
    lmp_stack_push (*l, lmp_stack_get (stack, i));

  for (; i < stack->si; i++)
    lmp_stack_push (*r, lmp_stack_get (stack, i));

  return 1;
}

lmp_stack_t * lmp_stack_copy (lmp_stack_t * stack) {
  lmp_stack_t * res;

  res = malloc (sizeof (lmp_stack_t));
  res->bs = stack->bs;
  res->si = stack->si;
  res->sp = (lmp_node_t **) malloc (ROUND (stack->si, stack->bs) * sizeof (lmp_node_t *));
  memcpy (res->sp, stack->sp, stack->si * sizeof (lmp_node_t *));

  return res;
}

void lmp_stack_dump (lmp_stack_t * stack) {
  int i;

  lmp_warning ("Stack @%.8X has %d elements, block size is %d\n", stack, stack->si, stack->bs);
  for (i = 0; i < stack->si; i++) {
    lmp_warning ("[%d]: %.8X\n", i, stack->sp [i]);
    lmp_print_tree ("-> ", "  ", stack->sp [i]);
  }
}
